#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.django.rattail.models.contact`` -- Models for 'contact' Extension
"""

from __future__ import absolute_import

from django.db import models

from rattail.django.rattail.models import Model, uuid_field


__all__ = ['PhoneNumber', 'EmailAddress',
           'Person', 'PersonPhoneNumber', 'PersonEmailAddress']


class ContactInfoManager(models.Manager):

    def __init__(self, parent_type):
        super(ContactInfoManager, self).__init__()
        self.parent_type = parent_type

    def get_query_set(self):
        qs = super(ContactInfoManager, self).get_query_set()
        qs = qs.filter(parent_type=self.parent_type)
        qs = qs.order_by('preference')
        return qs


class ContactInfo(Model):

    class Meta(Model.Meta):
        abstract = True

    uuid = uuid_field()
    parent_type = models.CharField(max_length=20)
    preference = models.IntegerField()
    type = models.CharField(max_length=15, blank=True, null=True)


class PhoneNumber(ContactInfo):
    """
    Represents a phone (or fax) number associated with a contactable entity.
    """

    class Meta(ContactInfo.Meta):
        abstract = True
        db_table = ContactInfo.prefix('phone_numbers')
        verbose_name = "Phone Number"
        verbose_name_plural = "Phone Numbers"

    number = models.CharField(max_length=20)

    def __repr__(self):
        return "<%s: %s>" % (self.__class__.__name__, self.number)

    def __unicode__(self):
        return unicode(self.number)


class EmailAddress(ContactInfo):
    """
    Represents an email address associated with a contactable entity.
    """

    class Meta(ContactInfo.Meta):
        abstract = True
        db_table = ContactInfo.prefix('email_addresses')
        verbose_name = "Email Address"
        verbose_name_plural = "Email Addresses"

    address = models.CharField(max_length=255)

    def __repr__(self):
        return "<%s: %s>" % (self.__class__.__name__, self.address)

    def __unicode__(self):
        return unicode(self.address)


class Person(Model):
    """
    Represents a real, living and breathing person.  (Or, at least was
    previously living and breathing, in the case of the deceased.)
    """

    class Meta(Model.Meta):
        db_table = Model.prefix('people')
        verbose_name_plural = "People"

    uuid = uuid_field()
    first_name = models.CharField(max_length=50, blank=True, null=True)
    last_name = models.CharField(max_length=50, blank=True, null=True)
    display_name = models.CharField(max_length=100, blank=True, null=True, verbose_name="Display Name")

    def __repr__(self):
        return "<Person: %s>" % self.display_name

    def __unicode__(self):
        return unicode(self.display_name or '')

    def save(self, *args, **kwargs):
        if not self.display_name:
            if self.first_name and self.last_name:
                self.display_name = self.first_name + ' ' + self.last_name
            elif self.first_name:
                self.display_name = self.first_name
            elif self.last_name:
                self.display_name = self.last_name
        super(Person, self).save(*args, **kwargs)


class PersonContactInfo(models.Model):
    """
    Base class for person contact models (phone number, etc.).
    """

    class Meta:
        abstract = True

    store = models.ForeignKey(Person, db_column='parent_uuid')
    objects = ContactInfoManager('Person')

    def save(self, *args, **kwargs):
        self.parent_type = 'Person'
        super(PersonContactInfo, self).save(*args, **kwargs)


class PersonPhoneNumber(PhoneNumber, PersonContactInfo):
    """
    Represents a phone (or fax) number associated with a :class:`Person`.
    """


class PersonEmailAddress(EmailAddress, PersonContactInfo):
    """
    Represents an email address associated with a :class:`Person`.
    """    
