#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.django.rattail.models.core`` -- Core Model Stuff
"""

from django.db import models
from django.conf import settings

import edbob


__all__ = ['Model', 'uuid_field']


class Model(models.Model):
    """
    Base class for all Rattail models.
    """

    class Meta:
        abstract = True
        app_label = 'rattail'
        managed = bool(getattr(settings, 'RATTAIL_DJANGO_MANAGED', True))

    @classmethod
    def prefix(cls, table_name):
        """
        Returns a possibly-prefixed version of ``table_name``.

        The name returned will depend on whether or not a prefix is necessary,
        based on whether the Rattail tables are held in the native Django
        database, or an externally-managed one.
        """

        managed = bool(getattr(settings, 'RATTAIL_DJANGO_MANAGED', True))
        if managed:
            table_name = 'rattail_' + table_name
        return table_name


class UUIDField(models.CharField):
    """
    "Custom" type for UUID fields.  This is actually only necessary so that the
    Django Admin site will include the UUID as a hidden field when rendering
    inline forms.

    See also `this ticket <https://code.djangoproject.com/ticket/13696>`_.
    """

    def contribute_to_class(self, cls, name):
        super(UUIDField, self).contribute_to_class(cls, name)
        cls._meta.has_auto_field = True
        cls._meta.auto_field = self


def uuid_field():
    """
    Convenience function which returns a ``uuid`` column for use as a table's
    primary key.
    """

    return UUIDField(max_length=32, default=edbob.get_uuid,
                     primary_key=True, verbose_name="UUID")
